/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"

#include "Config.h"
#include "ConfigurationDialog.h"
#include "FtpCommandDialog.h"
#include "FtpInterface.h"
#include "LogDialog.h"
#include "MainWindow.h"
#include "QuestionDialog.h"
#include "ScratchFileMonitor.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QApplication>

//____________________________________________
Application::Application( CommandLineArguments arguments ):
    BaseApplication( nullptr, arguments ),
    Counter( "Application" )
{}

//____________________________________________
bool Application::realizeWidget( void )
{
    Debug::Throw( "Application::realizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    // top widget
    mainWindow_ = new MainWindow();
    mainWindow_->setWindowTitle( "Qftp" );

    // scratch files
    scratchFileMonitor_ = new ScratchFileMonitor( this );
    connect( qApp, SIGNAL(aboutToQuit()), scratchFileMonitor_, SLOT(deleteScratchFiles()) );
    connect( &mainWindow_->interface(), SIGNAL(scratchFileCreated(File)), scratchFileMonitor_, SLOT(add(File)) );
    connect( &mainWindow_->interface(), SIGNAL(busy()), &mainWindow(), SLOT(busy()) );
    connect( &mainWindow_->interface(), SIGNAL(ready()), &mainWindow(), SLOT(idle()) );

    mainWindow_->centerOnDesktop();
    mainWindow_->show();

    emit configurationChanged();

    // connect from arguments
    CommandLineParser parser( commandLineParser( _arguments() ) );
    if( parser.hasOption( "--host" ) )
    {
        QString host( parser.option( "--host" ) );
        QString user( "anonymous" );

        // parse host and user
        const int position( host.indexOf( "@" ) );
        if( position >= 0 )
        {
            user = host.left( position );
            host = host.mid( position+1 );
        } else if( parser.hasOption( "--user" ) ) user = parser.option( "--user" );

        mainWindow_->connectToHost(
            host,
            parser.hasOption( "--port" ) ? parser.option( "--port" ).toInt() : 21,
            user,
            parser.hasOption( "--pasword" ) ? parser.option( "--password" ):QString(),
            parser.hasFlag( "--auto" ) );

    } else {

        QMetaObject::invokeMethod( &mainWindow_->toggleConnectionAction(), "trigger", Qt::QueuedConnection );

    }

    return true;

}

//____________________________________________
CommandLineParser Application::commandLineParser( CommandLineArguments arguments, bool ignoreWarnings ) const
{
    Debug::Throw( "Application::commandLineParser.\n" );
    CommandLineParser out( BaseApplication::commandLineParser() );

    out.setGroup( CommandLineParser::applicationGroupName );
    out.registerFlag( "--auto", tr( "automatically connect to selected host") );
    out.registerOption( "--host", tr( "string" ), tr( "select host to connect to") );
    out.registerOption( "--port", tr( "integer" ), tr( "set connection's port (default is 21)" ) );
    out.registerOption( "--user", tr( "string" ), tr( "set connection's user name (default is 'anonymous')") );
    out.registerOption( "--password", tr( "string" ), tr( "open files in a new window") );
    if( !arguments.isEmpty() ) out.parse( arguments, ignoreWarnings );
    return out;

}

//____________________________________________
void Application::usage( void ) const
{
    _usage( "qftp", tr( "[options] [host]" ) );
    commandLineParser().usage();
    return;
}

//___________________________________________________________
void Application::_configuration( void )
{
    Debug::Throw( "Application::_configuration.\n" );
    emit saveConfiguration();
    ConfigurationDialog dialog;
    connect( &dialog, SIGNAL(configurationChanged()), SIGNAL(configurationChanged()) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();
}

//________________________________________________
bool Application::_processCommand( Server::ServerCommand command )
{

    Debug::Throw( "Application::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    if( command.command() == Server::ServerCommand::Raise )
    {
        if( mainWindow_ ) mainWindow_->uniconify();

        CommandLineParser parser( commandLineParser( command.arguments() ) );
        if( parser.hasOption( "--host" ) )
        {
            QString host( parser.option( "--host" ) );
            QString user( "anonymous" );

            // parse host and user
            const int position( host.indexOf( "@" ) );
            if( position >= 0 )
            {
                user = host.left( position );
                host = host.mid( position+1 );
            } else if( parser.hasOption( "--user" ) ) user = parser.option( "--user" );

            const QString buffer = QString( tr( "Accept requested connection to host '%1' ?" ) ).arg( host );
            if( QuestionDialog( mainWindow_, buffer ).centerOnParent().exec() )
            {
                mainWindow_->connectToHost(
                    host,
                    parser.hasOption( "--port" ) ? parser.option( "--port" ).toInt() : 21,
                    user,
                    parser.hasOption( "--pasword" ) ? parser.option( "--password" ):QString(),
                    parser.hasFlag( "--auto" ) );
            }
        }

        return true;

    } else return false;

}
