/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConnectionDialog.h"
#include "ConnectionFrame.h"
#include "ConnectionParser.h"
#include "CustomComboBox.h"
#include "CustomDialog.h"
#include "GridLayout.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "QuestionDialog.h"
#include "SimpleListView.h"
#include "XmlOptions.h"

#include <QCheckBox>
#include <QHeaderView>
#include <QLabel>
#include <QLayout>
#include <QSplitter>

//______________________________________________________________________
ConnectionDialog::ConnectionDialog( QWidget* parent ):
    BaseDialog( parent ),
    Counter( "ConnectionDialog" )
{

    Debug::Throw( "ConnectionDialog::ConnectionDialog.\n" );

    setOptionName( "CONNECTION_DIALOG" );

    // vertical layout
    QVBoxLayout *layout( new QVBoxLayout() );
    layout->setMargin(10);
    layout->setSpacing(5);
    setLayout( layout );

    // horizontal layout
    QHBoxLayout* hLayout = new QHBoxLayout();
    hLayout->setMargin(0);
    hLayout->setSpacing(5);
    layout->addLayout( hLayout );

    // create list
    hLayout->addWidget( list_ = new SimpleListView( this ), 0 );
    list_->setProperty( "_kde_side_panel_view", true );
    list_->setModel( &model_ );

    // right widget
    QWidget* rightWidget;
    hLayout->addWidget( rightWidget = new QWidget( this ), 1 );

    // vertical layout
    QVBoxLayout* vLayout( new QVBoxLayout() );
    vLayout->setMargin(0);
    vLayout->setSpacing(5);
    rightWidget->setLayout( vLayout );

    // connection frame
    vLayout->addWidget( frame_ = new ConnectionFrame( rightWidget ) );

    // save password
    QCheckBox* checkbox = new QCheckBox( tr( "Remember password" ), frame_ );
    GridLayout* gridLayout = static_cast<GridLayout*>( frame_->layout() );

    gridLayout->addWidget( checkbox, gridLayout->currentRow()+1, 1, 1, 1 );
    checkbox->setChecked( XmlOptions::get().get<bool>( "SAVE_PASSWORDS" ) );
    connect( checkbox, SIGNAL(toggled(bool)), SLOT(_toggleSavePasswords(bool)) );

    // show password
    checkbox = new QCheckBox( tr( "Show password" ), frame_ );
    gridLayout->addWidget( checkbox, gridLayout->currentRow()+1, 1, 1, 1 );
    connect( checkbox, SIGNAL(toggled(bool)), SLOT(_toggleShowPassword(bool)) );

    vLayout->addStretch( 1 );

    // connect
    vLayout->addWidget( connectButton_ = new QPushButton( tr( "Connect" ), rightWidget ) );
    connect( connectButton_, SIGNAL(clicked()), SLOT(_connect()) );
    connect( connectButton_, SIGNAL(clicked()), SLOT(close()) );
    connectButton_->setToolTip( tr( "Connect" ) );
    connectButton_->setDefault( true );
    connectButton_->setIcon( IconEngine::get( IconNames::Connect ) );

    // buttons
    vLayout->addWidget( saveButton_ = new QPushButton( tr( "Save" ), rightWidget ) );
    connect( saveButton_, SIGNAL(clicked()), SLOT(_save()) );
    saveButton_->setToolTip( tr( "Save current settings" ) );
    saveButton_->setIcon( IconEngine::get( IconNames::Save ) );

    vLayout->addWidget( removeButton_ = new QPushButton( tr( "Remove" ), rightWidget ) );
    connect( removeButton_, SIGNAL(clicked()), SLOT(_remove()) );
    removeButton_->setToolTip( tr( "Remove current settings from list" ) );
    removeButton_->setIcon( IconEngine::get( IconNames::Remove ) );

    QPushButton *cancelButton;
    vLayout->addWidget( cancelButton = new QPushButton( tr( "Cancel" ), rightWidget ) );
    cancelButton->setShortcut( Qt::Key_Escape );
    connect( cancelButton, SIGNAL(clicked()), SLOT(_cancel()) );
    connect( cancelButton, SIGNAL(clicked()), SLOT(close()) );
    cancelButton->setToolTip( tr( "Discard changes and cancel connection" ) );
    cancelButton->setIcon( IconEngine::get( IconNames::DialogCancel ) );

    connect( list_->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(_select()) );
    connect( list_->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(_updateButtons()) );

    // direct connection on activation.
    connect( list_, SIGNAL(activated(QModelIndex)), SLOT(_select()) );
    connect( list_, SIGNAL(activated(QModelIndex)), SLOT(_connect()) );
    connect( list_, SIGNAL(activated(QModelIndex)), SLOT(close()) );

    // load connection attributes into list
    _load();

    // update button state
    _updateButtons();

    if( connectButton_->isEnabled() ) connectButton_->setFocus();

    adjustSize();

}

//______________________________________________________________________
void ConnectionDialog::closeEvent( QCloseEvent* event )
{
    Debug::Throw( "ConnectionDialog::closeEvent.\n" );

    // save to resources
    auto connections( model_.get() );
    auto index( list_->selectionModel()->currentIndex() );
    if( index.isValid() )
    {
        for( auto& connection:connections )
        { connection.setIsDefault( model_.index( connection ).row() == index.row() ); }
    }

    // get current selection
    parser_.set( connections );
    parser_.write( QString( XmlOptions::get().raw( "RC_FILE" ) ) );

    return BaseDialog::closeEvent( event );
}

//______________________________________________________________________
void ConnectionDialog::_load( void )
{
    Debug::Throw( "ConnectionDialog::_load.\n" );

    // read from options
    parser_.read( QString( XmlOptions::get().raw( "RC_FILE" ) ) );

    // store in list
    model_.set( parser_.attributes() );

    // select last item
    if( !parser_.attributes().empty() )
    {

        for( const auto& attributes:parser_.attributes() )
        {
            if( attributes.isDefault() )
            {

                // select default item
                QModelIndex index( model_.index( attributes ) );
                list_->selectionModel()->select( index, QItemSelectionModel::SelectCurrent|QItemSelectionModel::Rows );
                list_->selectionModel()->setCurrentIndex( index, QItemSelectionModel::Select|QItemSelectionModel::Rows );

                // assign to frame
                frame_->load( attributes );
                break;
            }
        }


    }

}

//______________________________________________________________________
void ConnectionDialog::_select( void )
{
    Debug::Throw( "ConnectionDialog::_select.\n" );
    QModelIndexList indexes( list_->selectionModel()->selectedIndexes() );
    if( !indexes.empty() ) frame_->load( model_.get(indexes.front()) );
}

//______________________________________________________________________
void ConnectionDialog::_remove( void )
{

    Debug::Throw( "ConnectionDialog::_Remove.\n" );

    // retrieve selection
    ConnectionAttributes::List selection( model_.get( list_->selectionModel()->selectedRows() ) );
    Q_ASSERT( selection.size() == 1 );

    // confirm
    const QString buffer = QString( tr( "Remove connection named '%1' from list ?" ) ).arg( selection.begin()->name() );
    if( !QuestionDialog( this, buffer ).exec() ) return;

    model_.remove( *selection.begin() );

}

//______________________________________________________________________
void ConnectionDialog::_save( void )
{

    Debug::Throw( "ConnectionDialog::_save.\n" );
    CustomDialog dialog( this );
    CustomComboBox* combobox;

    dialog.mainLayout().addWidget( new QLabel( tr( "Connection name:" ), &dialog ) );
    dialog.mainLayout().addWidget( combobox  = new CustomComboBox( &dialog ) );
    combobox->setEditable( true );
    combobox->setAutoCompletion( true, Qt::CaseSensitive );

    // get model, and data; populate combobox
    for( const auto& attributes:model_.get() )
    { combobox->addItem( attributes.name() ); }

    // select current item
    QModelIndex index( list_->selectionModel()->currentIndex() );
    if( index.isValid() )
    { combobox->setCurrentIndex( combobox->findText( model_.get( index ).name() ) ); }

    dialog.adjustSize();
    dialog.centerOnParent();

    if( dialog.exec() == QDialog::Rejected ) return;
    model_.add( frame_->attributes().setName( combobox->currentText() ) );

}

//______________________________________________________________________
void ConnectionDialog::_connect( void )
{
    Debug::Throw( "ConnectionDialog::_connect.\n" );

    // update password if changed
    QModelIndex index( list_->selectionModel()->currentIndex() );
    if( index.isValid() && XmlOptions::get().get<bool>( "SAVE_PASSWORDS" ) )
    {

        // get current attributes
        ConnectionAttributes current( frame_->attributes() );

        // get selected attributes
        ConnectionAttributes selected( model_.get( index ) );

        // compare
        if( current == selected && current.password() != selected.password() )
        {
            selected.setPassword( current.password() );
            model_.add( selected );
        }
    }

    // end
    done( QDialog::Accepted );

}

//______________________________________________________________________
void ConnectionDialog::_cancel( void )
{
    Debug::Throw( "ConnectionDialog::_cancel.\n" );
    done( QDialog::Rejected );
}

//______________________________________________________________________
void ConnectionDialog::_toggleSavePasswords( bool state )
{
    Debug::Throw( "ConnectionDialog::_toggleSavePassword.\n" );
    XmlOptions::get().set<bool>( "SAVE_PASSWORDS", state );
}

//______________________________________________________________________
void ConnectionDialog::_toggleShowPassword( bool value )
{ frame_->passwordEditor().setEchoMode( value ? QLineEdit::Normal:QLineEdit::Password); }

//______________________________________________________________________
void ConnectionDialog::_updateButtons( void )
{
    Debug::Throw( "ConnectionDialog::_updateButtons.\n" );
    removeButton_->setEnabled( !list_->selectionModel()->selectedRows().empty() );
}
