#ifndef FtpCommand_h
#define FtpCommand_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "FileInfo.h"

#include <QFtp>
#include <QList>
#include <QHash>

class FtpCommand: public Counter
{

    public:

    //* enumeration for command type
    enum Type
    {
        None,
        ConnectToHost,
        Login,
        Close,
        ChangeDirectory,
        CreateDirectory,
        ShowDirectory,
        Copy,
        Rename,
        Remove
    };

    //* list of commands
    using List = QList<FtpCommand>;

    //* constructor
    FtpCommand( Type = None );

    //* equal to operator
    bool operator == (const FtpCommand& command ) const
    { return id_ == command.id_; }

    //* less than to operator
    bool operator < (const FtpCommand& command ) const
    { return id_ < command.id_; }

    //*@name accessors
    //@{

    //* type
    Type type( void ) const
    { return type_; }

    //* return command name
    QString name( void ) const;

    //* id
    int id( void ) const
    { return id_; }

    //* ftp id
    int ftpId( void ) const
    { return ftpId_; }

    //* argument size
    int argumentCount( void ) const
    { return arguments_.size(); }

    //* get argument
    const FileInfo& argument( int index ) const
    { return arguments_[index]; }

    //@}

    //*@name modifiers
    //@{

    //* id
    void setFtpId( const int& ftpId )
    { ftpId_ = ftpId; }

    //* add argument
    FtpCommand& addArgument( const FileInfo& argument )
    {
        arguments_ << argument;
        return *this;
    }

    //* push arguments
    FtpCommand& operator << (const FileInfo& argument )
    { return addArgument( argument ); }

    //* clear arguments
    void clearArguments( void )
    { arguments_.clear(); }

    //@}

    //* retrieve command with same type
    class SameTypeFTor
    {

        public:

        //* constructor
        SameTypeFTor( const Type& type ):
            type_( type )
        {}

        //* predicate
        bool operator() (const FtpCommand& command ) const
        { return command.type_ == type_; }

        private:

        //* predicted id
        Type type_;

    };

    private:

    //* static id counter
    static unsigned int& _counter( void );

    //* command names
    using NameMap = QHash< Type, QString >;

    //* command names
    static NameMap& _names( void );

    //* command names
    static void _initializeNames( void );

    //* command id
    int id_ = -1;

    //* ftp id
    int ftpId_ = 0;

    //* command type
    Type type_ = None;

    //* arguments
    QList<FileInfo> arguments_;

};

#endif
