/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "FtpMacroModel.h"

#include "CustomPixmap.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "IconSize.h"
#include "TimeStamp.h"
#include "XmlOptions.h"

#include <QIcon>

//_______________________________________________
const QString FtpMacroModel::columnTitles_[ FtpMacroModel::nColumns ] =
{
    tr( "Id" ),
    tr( "Name" ),
    tr( "File" )
};

//_______________________________________________________________
FtpMacroModel::FtpMacroModel( QObject* parent ):
    ListModel<FtpMacro>( parent ),
    Counter( "FtpMacroModel" )
{

    Debug::Throw( "FtpMacroModel::FtpMacroModel.\n" );
    updateConfiguration();

}

//__________________________________________________________________
QVariant FtpMacroModel::data( const QModelIndex& index, int role ) const
{

    // check index
    if( !contains( index ) ) return QVariant();

    // retrieve associated macro
    const FtpMacro& macro( get()[index.row()] );

    // return text associated to macro and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {

            case Id: return macro.id();
            case Name: return macro.name();
            case Filename: return macro.argumentCount() > 0  ? macro.argument(0).file():QVariant();
            default: return QVariant();
        }
    } else if( role == Qt::DecorationRole && index.column() == Name ) {

        // return icon associated to file
        return _icon(macro.type());

    }

    return QVariant();

}

//__________________________________________________________________
QVariant FtpMacroModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//_________________________________________________________
void FtpMacroModel::updateConfiguration( void )
{

    Debug::Throw( "FtpMacroModel::updateConfiguration.\n" );

    beginResetModel();

    // reinitialize icons
    _icons().clear();
    _installIcons();

    endResetModel();

}

//_____________________________________________________________________
void FtpMacroModel::_installIcons( void ) const
{

    Debug::Throw( "FtpMacroList::_installIcons" );

    if( !_icons().empty() ) return;

    // type icons
    using IconNamesHash = QHash< FtpMacro::Type, QString >;
    IconNamesHash typeNames;
    typeNames[FtpMacro::Connect] = IconNames::Connect;
    typeNames[FtpMacro::Add] = IconNames::Add;
    typeNames[FtpMacro::Clear] = IconNames::Delete;
    typeNames[FtpMacro::Abort] = IconNames::Abort;
    typeNames[FtpMacro::CopyRecursive] = IconNames::Paste;
    typeNames[FtpMacro::CreateDirectory] = IconNames::CreateDirectory;
    typeNames[FtpMacro::Rename] = IconNames::Rename;
    typeNames[FtpMacro::Remove] = IconNames::Delete;
    typeNames[FtpMacro::Open] = IconNames::Edit;
    typeNames[FtpMacro::ShowDirectory] = IconNames::Find;
    typeNames[FtpMacro::ListRecursive] = IconNames::Find;

    // load link pixmap
    for( auto&& iter = typeNames.begin(); iter != typeNames.end(); ++iter )
    { _icons().insert( iter.key(), IconEngine::get( iter.value() ) ); }

    return;
}

//_______________________________________________
FtpMacroModel::IconCache& FtpMacroModel::_icons( void )
{
    static IconCache cache;
    return cache;
}

//____________________________________________________________
const QIcon& FtpMacroModel::_icon( FtpMacro::Type type )
{

    IconCache::iterator iter = ( _icons().find( type ) );
    if( iter != _icons().end() ) return iter.value();
    else return _icons().insert( type, CustomPixmap( IconSize( IconSize::Medium ), CustomPixmap::Transparent ) ).value();

}

//________________________________________________________
bool FtpMacroModel::SortFTor::operator () ( FtpMacro first, FtpMacro second ) const
{

    if( order_ == Qt::DescendingOrder ) std::swap( first, second );
    switch( type_ )
    {

        case FtpMacroModel::Name:
        if( first.type() != second.type() ) return first.type() < second.type();
        else break;

        case FtpMacroModel::Filename:
        {
            const FileInfo source( first.argumentCount() > 0 ? first.argument(0).file():FileInfo() );
            const FileInfo destination( second.argumentCount() > 0 ? second.argument(0).file():FileInfo() );
            if( source.file() != destination.file() ) return source.file() < destination.file();
            else break;
        }

        default: break;
    }

    return first.id() < second.id();

}
